@@ -3,9 +3,9 @@ package com.sun.tools.javac.resources;
 public final class version extends java.util.ListResourceBundle {
     protected final Object[][] getContents() {
         return new Object[][] {
-            { "full", "16.0.1+9-24" },
-            { "jdk", "16.0.1" },
-            { "release", "16.0.1" },
+            { "full", "18.0.2.1+1" },
+            { "jdk", "18.0.2.1" },
+            { "release", "18.0.2.1" },
         };
     }
 }
